@@ -76,13 +76,13 @@ class SubscriptionManager:
             await self._init_web_socket()
         except Exception:  # pylint: disable=broad-except
             _LOGGER.debug("Failed to connect. Reconnecting... ", exc_info=True)
-            self.retry()
+            await self.retry()
             return
 
         k = 0
         while self._state in (
-                STATE_RUNNING,
-                STATE_STARTING,
+            STATE_RUNNING,
+            STATE_STARTING,
         ):
             try:
                 msg = await asyncio.wait_for(self.websocket.recv(), timeout=60)
@@ -90,13 +90,13 @@ class SubscriptionManager:
                 k += 1
                 if k > 10:
                     _LOGGER.debug("No data, reconnecting.")
-                    self.retry()
+                    await self.retry()
                     return
                 _LOGGER.debug("No websocket data, sending a ping.")
                 await asyncio.wait_for(await self.websocket.ping(), timeout=20)
             except Exception:  # pylint: disable=broad-except
                 if self._state == STATE_RUNNING:
-                    self.retry()
+                    await self.retry()
             else:
                 k = 0
                 self._process_msg(msg)
@@ -108,7 +108,7 @@ class SubscriptionManager:
 
         start_time = time()
 
-        for subscription_id in self.subscriptions.copy().keys():
+        for subscription_id in self.subscriptions.copy():
             _LOGGER.debug("Sending unsubscribe: %s", subscription_id)
             await self.unsubscribe(subscription_id)
 
@@ -124,11 +124,13 @@ class SubscriptionManager:
         self._cancel_client_task()
         _LOGGER.debug("Server connection is stopped")
 
-    def retry(self):
+    async def retry(self):
         """Retry to connect to websocket."""
         _LOGGER.debug("Retry, state: %s", self._state)
         self._cancel_retry_timer()
         self._state = STATE_STARTING
+        _LOGGER.debug("Close websocket")
+        await self._close_websocket()
         _LOGGER.debug("Restart")
         self._retry_timer = self.loop.call_soon(self.start)
         _LOGGER.debug("Reconnecting to server.")
@@ -219,6 +221,7 @@ class SubscriptionManager:
 
     async def _init_web_socket(self):
         self.websocket = await asyncio.wait_for(
+            # pylint: disable=no-member
             websockets.connect(
                 self._url,
                 subprotocols=["graphql-transport-ws"],
